@@ -51,13 +51,14 @@ from hubzero.mw.log import setup_log, log
 
 #
 # This config file can override any of the following global constants.
+# Policy: default to secure settings if possible
 #
 EXEC_PROXY_CONFIG_FILE = '/etc/mw-proxy/exec-proxy.conf'
 EXEC_CONFIGDIR = '/etc/mw-proxy'
 
 EXEC_LISTEN_HOST = None    # IP address to bind to
 EXEC_LISTEN_PORT = None    # port number to bind to
-EXEC_LISTEN_SSL = None     # Use SSL for listening socket? (True/False)
+EXEC_LISTEN_SSL = True     # Use SSL for listening socket? (True/False)
 EXEC_SERVER_KEY = None     # SSL key
 EXEC_SERVER_CERT = None    # SSL certificate
 EXEC_SERVER_CACERT = None  # SSL CA certificate
@@ -67,10 +68,15 @@ PIDFILE = '/var/run/exec-proxy.pid'
 PROXY_LOG = '/var/log/exec-proxy/exec-proxy.log'
 HEADER_TIMEOUT = 20
 
+# VIRT_ENGINE must be 'docker or 'openvz'
+VIRT_ENGINE = 'openvz'
+# DOCKERTOOL_NET should be something like 172.16.8.0 and match the configuration for container.py
+DOCKERTOOL_NET = '0.0.0.0'
 FORWARD_PORT = 8000
 RUN_UID = 'hz-exec-proxy'
 RUN_GID = 'hz-exec-proxy'
-VERBOSITY = 0
+VERBOSITY = 5
+LISTEN_TLSv1_2 = True
 
 from hubzero.mw.constants import MYSQL_CONN_ATTEMPTS, EXEC_CONFIG_FILE
 from hubzero.mw.constants import MYSQL_CONN_ATTEMPTS
@@ -80,6 +86,8 @@ if os.path.isfile(EXEC_PROXY_CONFIG_FILE):
 else:
   print "configuration file not found, proceeding with defaults"
 
+# choose SSL.TLSv1_2_METHOD (required in 2018) or SSL.TLSv1_METHOD (very old versions of Python)
+# probably should be a configurable option
 verbosity = int(VERBOSITY)
 
 #==============================================================================
@@ -142,7 +150,14 @@ def create_listener(host,port):
   sock = socket.socket()
   sock.setsockopt(socket.SOL_SOCKET,socket.SO_REUSEADDR,1)
   if EXEC_LISTEN_SSL:
-    ctx = SSL.Context(SSL.TLSv1_METHOD)
+    if LISTEN_TLSv1_2:
+      ctx = SSL.Context(SSL.TLSv1_2_METHOD)
+      ctx.set_options(SSL.OP_NO_TLSv1)
+      ctx.set_options(SSL.OP_NO_TLSv1_1)
+    else:
+      ctx = SSL.Context(SSL.TLSv1_METHOD)
+    ctx.set_options(SSL.OP_NO_SSLv2)
+    ctx.set_options(SSL.OP_NO_SSLv3)
     ctx.use_privatekey_file (os.path.join(EXEC_CONFIGDIR,EXEC_SERVER_KEY))
     ctx.use_certificate_file(os.path.join(EXEC_CONFIGDIR,EXEC_SERVER_CERT))
     if EXEC_SERVER_CACERT != None:
@@ -151,8 +166,6 @@ def create_listener(host,port):
       ctx.load_tmp_dh(os.path.join(EXEC_CONFIGDIR,EXEC_SERVER_DHPARAM))
     if EXEC_SERVER_CIPHERS != None:
       ctx.set_cipher_list(EXEC_SERVER_CIPHERS)
-    ctx.set_options(SSL.OP_NO_SSLv2)
-    ctx.set_options(SSL.OP_NO_SSLv3)
     ctx.set_options(SSL.OP_SINGLE_DH_USE)
     sock = SSL.Connection(ctx, sock)
   sock.bind((host, port))
@@ -362,12 +375,23 @@ def verify_cb(conn, crt, errnum, depth, ok):
 def address_of_container(ct):
   try:
     machine_number = int(socket.gethostbyname(socket.getfqdn()).split('.')[3])
-    digit = ct/255 + (machine_number % 64) * 4
-    ip = CONTAINER_CONF['PRIVATE_NET'] % (digit % 256, ct %255)
+    digit = ct/256 + (machine_number % 64) * 4
+    ip = CONTAINER_CONF['PRIVATE_NET'] % (digit % 256, ct %256)
     return ip
   except:
     fatal('Unable to determine machine number')
 
+def address_of_docker(ct):
+  try:
+    if DOCKERTOOL_NET == '0.0.0.0':
+      fatal('DOCKERTOOL_NET configuration variable is undefined, please edit the configuration file for the exec-proxy service')
+    gwbytes = DOCKERTOOL_NET.split('.')
+    IPa = gwbytes[0:2] + [str(int(gwbytes[2]) + ct/256), str(ct %256)]
+    log('tool container IP is ' + ".".join(IPa))
+    return ".".join(IPa)
+  except:
+    fatal('Unable to determine tool network IP from ' + str(ct))
+
 #==============================================================================
 # Interpret the header and open a socket/SSL to the execution host's proxy.
 #==============================================================================
@@ -396,8 +420,12 @@ def find_target(hdr):
   except:
     fatal('Malformed URL: (comp3) ' + url)
 
-  # The last octet of the address to connect to is the container number.
-  host = address_of_container(ct)
+  if VIRT_ENGINE == 'openvz':
+    host = address_of_container(ct)
+  elif VIRT_ENGINE == 'docker':
+    host = address_of_docker(ct)
+  else:
+    fatal('VIRT_ENGINE has an unsupported value, please edit the configuration file')
 
   return host
 
